﻿package 
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.PixelSnapping;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.media.Camera;
	import flash.media.Video;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.utils.ByteArray;
	
	import org.libspark.flartoolkit.core.FLARCode;
	import org.libspark.flartoolkit.core.FLARSquare;
	import org.libspark.flartoolkit.core.param.FLARParam;
	import org.libspark.flartoolkit.core.raster.rgb.FLARRgbRaster_BitmapData;
	import org.libspark.flartoolkit.core.transmat.FLARTransMatResult;
	import org.libspark.flartoolkit.detector.FLARMultiMarkerDetector;
	import org.libspark.flartoolkit.pv3d.FLARBaseNode;
	import org.libspark.flartoolkit.pv3d.FLARCamera3D;
	import org.papervision3d.lights.PointLight3D;
	import org.papervision3d.materials.BitmapFileMaterial;
	import org.papervision3d.materials.BitmapMaterial;
	import org.papervision3d.materials.shadematerials.FlatShadeMaterial;
	import org.papervision3d.objects.primitives.Cube;
	import org.papervision3d.objects.primitives.Plane;
	import org.papervision3d.render.LazyRenderEngine;
	import org.papervision3d.scenes.Scene3D;
	import org.papervision3d.view.Viewport3D;
	
	[SWF(width=640, height=480, backgroundColor=0xffffff, frameRate=60)]
	
	/**
	 * ...
	 * @author Mikko Haapoja
	 */
	public class Main extends Sprite 
	{
		/*
		 * camera_para.dat is a Binary file created by --PROGRAM NAME HERE--
		 * Basically what it does is give information for FlarToolkit to
		 * correct distortion and stuff like that, that your camera does.
		 * It will also contain a Projection Matrix
		 */
		[Embed(source="../lib/camera_para.dat", mimeType="application/octet-stream")]
		private var CameraParameters:Class;
		 
		/* the four CHI markers */
		[Embed(source="../lib/newmarker1.pat", mimeType="application/octet-stream")]
		private var MarkerPattern1:Class;
		[Embed(source="../lib/newmarker2.pat", mimeType="application/octet-stream")]
		private var MarkerPattern2:Class;
		[Embed(source="../lib/newmarker3.pat", mimeType="application/octet-stream")]
		private var MarkerPattern3:Class;
		[Embed(source="../lib/newmarker4.pat", mimeType="application/octet-stream")]
		private var MarkerPattern4:Class;

		[Embed(source="../lib/skydive.jpg", mimeType="image/jpg")]
		private var Image1:Class;
		[Embed(source="../lib/3ladies.jpg", mimeType="image/jpg")]
		private var Image2:Class;
		[Embed(source="../lib/classroomkids.jpg", mimeType="image/jpg")]
		private var Image3:Class;
		[Embed(source="../lib/gamekid.jpg", mimeType="image/jpg")]
		private var Image4:Class;
		[Embed(source="../lib/weAreHCI.png", mimeType="image/png")]
		private var Image5:Class;

		private var MarkerPattern8:Class;
		private var MarkerPattern:Array;
		private var numMarkers:int = 4;
		private var markerWidth:Array;
		private var markerStuff:Array;
		private var markerLag:Array;
				
		private var printDebug:Boolean = false;
		private var debug:TextField;

		private var cameraParameters:FLARParam;
		private var markerPattern:Array;
		private var markerPatternTemp:FLARCode;
		private var raster:FLARRgbRaster_BitmapData;
		private var detector:FLARMultiMarkerDetector;
		
		private var cam:Camera;
		private var vid:Video;
		private var capture:Bitmap;//Data;
		
		private var cam3D:FLARCamera3D;
		private var scene3D:Scene3D;
		private var viewPort:Viewport3D;
		private var mainContainer:FLARBaseNode;
		private var renderer:LazyRenderEngine;
		private var planeMat:FlatShadeMaterial;
		private var numShot:uint = 0;
		
		private var trans:FLARTransMatResult;
		private var prevSet:Boolean = false;
		private var prevZ:Number = 0;
		
		private var cube:Cube;
		private var bitmap:BitmapData;
		private var material:BitmapFileMaterial;
					
		public function Main():void 
		{
			/*
			 * cameraParameters will hold our parameters from the camera noted above.
			 * Flar will use the camera parameters to make your 3d stuff
			 * look right
			 */
			cameraParameters = new FLARParam();
			cameraParameters.loadARParam(new CameraParameters() as ByteArray);
			cameraParameters.changeScreenSize(320,240);
			/*
			 * markerPattern will hold the data from our pattern files for Flar
			 * to use to look for our Markers
			 */
			if (numMarkers == 4)
			{
				MarkerPattern = new Array(new MarkerPattern1(), new MarkerPattern2(), new MarkerPattern3(), new MarkerPattern4());
			} else {
				//  NO NO NO!
			}

			markerWidth = new Array(numMarkers);
			markerPattern = new Array(numMarkers);
			markerStuff = new Array(numMarkers);
			markerLag = new Array(numMarkers);
			for(var x:int = 0; x<numMarkers; x++)	
			{
				markerLag[x] = 0;
				markerWidth[x] = 80;		
				markerStuff[x] = new FLARBaseNode();  // to hold the stuff for each marker		
				markerPattern[x] = new FLARCode(8, 8);
				markerPattern[x].loadARPatt(MarkerPattern[x]);
			}
			
			//Get our webcam going
			cam = Camera.getCamera();
			//Set the webcam to run as 320x240 at 30 frames per second
			cam.setMode(320, 240, 30);
			
			//Create a new video object to show our webcam
			vid = new Video(320, 240);
			vid.attachCamera(cam);
			addChild(vid);
			
			/*
			 * capture will hold BitmapData of what is shown on the 
			 * webcam FlarToolkit will use this bitmapdata to look
			 * for our mark pattern
			 * 
			 * we will need to redraw vid to capture every frame
			 */
			capture = new Bitmap(new BitmapData(320, 240, false, 0x0), PixelSnapping.AUTO, true);
			capture.bitmapData.draw(vid);
			
			/*
			 * raster will hold our BitmapData for Flar to use
			 */
			raster = new FLARRgbRaster_BitmapData(capture.bitmapData);
			detector = new FLARMultiMarkerDetector(cameraParameters, markerPattern, markerWidth, numMarkers);
			
			/*
			 * cam3d is a Papervision Cam3D
			 * and will be our 3d camera that is setup
			 * from our camera data file imported above
			 */
			cam3D = new FLARCamera3D(cameraParameters);
			
			//Papervision scence
			scene3D = new Scene3D();
			
			/*
			 * This is our mainContainer what will happen
			 * is that Flar will give us a transformation
			 * matrix based on the marker and we can use
			 * that transformation matrix to rotate the
			 * mainContainer
			 */
			mainContainer = new FLARBaseNode();
			scene3D.addChild(mainContainer);
			
			/*
			 * This is our Papervision viewport
			 * This is a gotcha... I'm not sure why the viewPort needs
			 * to be the same size as the video but then scaled up twice
			 * even still then it's a bit offset. I didn't feel like doing
			 * more offsetting because I don't like that kind of stuff
			 * 
			 * Maybe we can figure this one out together
			 */
			viewPort = new Viewport3D(320, 240);
//			viewPort.scaleX = 2;
//			viewPort.scaleY = 2;
			viewPort.x = -4;
			addChild(viewPort);

			// mirror the video
			this.scaleX = 1.6;
			this.scaleY = 1.6;
			//this.x = 400;

			//Our papervision renderer
			renderer = new LazyRenderEngine(scene3D, cam3D, viewPort);
			
			//Papervision light
			var light:PointLight3D = new PointLight3D();
			light.x = 1000;
			light.y = 1000;
			light.z = -1000;
			
			// Set some colors so we can see if loading is still happening,
			// or if it failed
			BitmapFileMaterial.LOADING_COLOR = 0x0000FF;
			BitmapFileMaterial.ERROR_COLOR = 0xFF0000;

/*							
			var cubeMaterialList:MaterialsList = new MaterialsList( { all: new FlatShadeMaterial(light, 0x0099FF, 0x0066AA),
						front: new BitmapFileMaterial("../lib/3 ladies 90dpi.jpg")	
			 } );
			cube = new Cube(cubeMaterialList,
									 80,
									 10,
									 80);
			
			cube.z += 5;
			cube.rotationZ = -90;

			mainContainer.addChild(cube);
*/			
			if (printDebug) {
				// debugging info
				debug = new TextField();
				//debug.x = 10;
				//debug.y = 10;
				debug.autoSize = TextFieldAutoSize.LEFT;
				debug.width = 640;
				//debug.height = 100;
				debug.wordWrap = true;
				debug.multiline = true;
				debug.text = "<0>";
				addChild(debug);			
			}

			//Material for our plane
			//var pic:BitmapFileMaterial = new BitmapFileMaterial("http://www.chi2010.org/AR/3 ladies 90dpi.jpg");	
			var pic:BitmapMaterial = new BitmapMaterial((new(Image1) as Bitmap).bitmapData);	
			pic.doubleSided = true;
			var plane:Plane = new Plane(pic, 80, 80);
			plane.rotationZ = -90;
			markerStuff[0].addChild(plane);
			mainContainer.addChild(markerStuff[0]);

			var hcipic:BitmapMaterial = new BitmapMaterial((new(Image5) as Bitmap).bitmapData);	
			hcipic.doubleSided = true;
			var hciplane:Plane = new Plane(hcipic, 80, 80);
			hciplane.rotationZ = -90;
			hciplane.x += 80;
			hciplane.z += 2;
			hciplane.scaleX = -1;
			markerStuff[0].addChild(hciplane);

			//Material for our plane
			//pic = new BitmapFileMaterial("http://chi2010.org/AR/classroom kids90dpi.jpg");	
			pic = new BitmapMaterial((new(Image2) as Bitmap).bitmapData);	
			pic.doubleSided = true;
			plane = new Plane(pic, 80, 80);
			plane.rotationZ = -90;
			markerStuff[1].addChild(plane);
			mainContainer.addChild(markerStuff[1]);

			hciplane = new Plane(hcipic, 80, 80);
			hciplane.rotationZ = -90;
			hciplane.x += 80;
			hciplane.z += 2;
			hciplane.scaleX = -1;
			markerStuff[1].addChild(hciplane);

			//Material for our plane
			//pic = new BitmapFileMaterial("http://chi2010.org/AR/Game Kid 90dpi.jpg");	
			pic = new BitmapMaterial((new(Image3) as Bitmap).bitmapData);	
			pic.doubleSided = true;
			plane = new Plane(pic, 80, 80);
			plane.rotationZ = -90;
			markerStuff[2].addChild(plane);
			mainContainer.addChild(markerStuff[2]);

			hciplane = new Plane(hcipic, 80, 80);
			hciplane.rotationZ = -90;
			hciplane.x += 80;
			hciplane.z += 2;
			hciplane.scaleX = -1;
			markerStuff[2].addChild(hciplane);

			//Material for our plane
			//pic = new BitmapFileMaterial("http://chi2010.org/AR/Skydive90dpi.jpg");	
			pic = new BitmapMaterial((new(Image4) as Bitmap).bitmapData);	
			pic.doubleSided = true;
			plane = new Plane(pic, 80, 80);
			plane.rotationZ = -90;
			markerStuff[3].addChild(plane);
			mainContainer.addChild(markerStuff[3]);

			hciplane = new Plane(hcipic, 80, 80);
			hciplane.rotationZ = -90;
			hciplane.x += 80;
			hciplane.z += 2;
			hciplane.scaleX = -1;
			markerStuff[3].addChild(hciplane);

			/*
			 * This is a Transformation matrix which Flar will
			 * fill and then we will use this to rotate mainContainer
			 */
			trans = new FLARTransMatResult();
			
			//Main loop where all the magic happens
			this.addEventListener(Event.ENTER_FRAME, mainEnter);
		}
		
		private function mainEnter(e:Event):void 
		{
			/*
			* Draw the current video screen to our capture
			* Flar will use this to figure out where our marker is
			*/
			capture.bitmapData.draw(vid);
			
			/*
			 * detector.detectMarkerLite(raster, 80) <-if marker found
			 * detector.getConfidence() <-how confident flar is in its find
			 */
			var useMarker:int = -1;
			var conf:Number = 0.0;
			var x:int;
			
			var numFound:int = detector.detectMarkerLite(raster, 80);
			if (printDebug) {
				debug.text = "<" + numFound + ">\n";				
			}

			for(x=0;x<numMarkers; x++){
	 			//debug.appendText("marker" + x.toString() + " " + markerLag[x] + "\n");
				if (markerLag[x] > 0)
					markerLag[x] -= 1;
				if (markerLag[x] == 0) {
					markerStuff[x].visible = false;
				}
			}
			
/*			for(x=0;x<numFound; x++){
				if (detector.getConfidence(x) > 0.8 && detector.getConfidence(x) > conf) {
					conf = detector.getConfidence(x);
					useMarker = x;
				}
			}
*/			
			for(x=0;x<numFound; x++){
				if (printDebug) {
				  debug.appendText(x.toString() + " (" + detector.getARCodeIndex(x).toString() + "):" +
			   			  				 detector.getConfidence(x).toString() + ": ");
				  var sq:FLARSquare = detector.getResult(x).square;
				  debug.appendText(sq.imvertex[0].toString() + "\n");
				}
			  if (detector.getConfidence(x) > 0.8) {			  
				  var code:int = detector.getARCodeIndex(x);
				  if (code >= 0 && code < numMarkers) {  	
					  //Get the transfomration matrix for the current marker position
 				 	  detector.getTransmationMatrix(x, trans);
		
					  //Translates and rotates the mainContainer so it looks right
					  markerStuff[code].setTransformMatrix(trans);
				  	  markerLag[code] = 4;
				  	  markerStuff[code].visible = true;
			      }
			   }
			}
			//Render the papervision scene
			renderer.render();
		}
	}
	
}